-- ----------------
-- CLOB-SPEICHERUNG
-- ----------------

--- Tabelle anlegen

DROP TABLE GemeindePOIs;

CREATE TABLE GemeindePOIs (
  gkz       INTEGER,                    -- Gemeindeschlssel
  poi_xml   SYS.XMLTYPE,                -- POIs in XML
  CONSTRAINT pk_gpoi PRIMARY KEY (gkz), -- Definition Primrschlssel
  CONSTRAINT fk_gpoi_gkz FOREIGN KEY (gkz) REFERENCES Gemeinden(gkz) 
                                        -- Fremdschlssel
);

--- Daten einfgen

INSERT INTO GemeindePOIs (gkz) VALUES (3403000);

UPDATE GemeindePOIs SET poi_xml = SYS.XMLTYPE.createXML
('<?xml version="1.0"?>
  <poi-set>
    <poi>
      <name>Schloss</name>
      <Point> <pos>8.21271 53.13930</pos> </Point>
    </poi>
    <poi>
      <name>FH</name>
      <Point> <pos>8.19974 53.14253</pos> </Point>
    </poi>
    <poi>
      <name>Bahnhof</name>
      <Point> <pos>8.22025 53.14416</pos> </Point>
    </poi>
  </poi-set>'
) WHERE gkz = 3403000;

COMMIT;

SET LONG 2000

-- XML abfragen:
SELECT gkz, poi_xml FROM GemeindePOIs;


-- -------------------------
-- STRUKTURIERTE SPEICHERUNG
-- -------------------------

-- Eine strukturierte Speicherung ist in Oracle 9.0 nicht mglich.
-- Es werden die dargestellten Anweisungen fr eine CLOB-Speicherung ausgefhrt.

--- Tabelle anlegen

DROP TABLE GemeindePOIsStr;

CREATE TABLE GemeindePOIsStr (
  gkz       INTEGER,                     -- Gemeindeschlssel
  poi_xml   SYS.XMLTYPE,                 -- POIs in XML
  CONSTRAINT pk_gpois PRIMARY KEY (gkz), -- Definition Primrschlssel
  CONSTRAINT fk_gpois_gkz FOREIGN KEY (gkz) REFERENCES Gemeinden(gkz) 
                                         -- Fremdschlssel
);

--- Daten einfgen

INSERT INTO GemeindePOIsStr (gkz,poi_xml) VALUES (3403000,SYS.XMLTYPE.createXML
('<?xml version="1.0"?>
  <poi-set xmlns="http://www.geodbs.de/xml/gemeindepoi"
           xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
           xsi:schemaLocation="http://www.geodbs.de/xml/gemeindepoi http://www.geodbs.de/xml/gemeindepoi.xsd">
    <poi>
      <name>Schloss</name>
      <Point> <pos>8.21271 53.13930</pos> </Point>
    </poi>
    <poi>
      <name>FH</name>
      <Point> <pos>8.19974 53.14253</pos> </Point>
    </poi>
    <poi>
      <name>Bahnhof</name>
      <Point> <pos>8.22025 53.14416</pos> </Point>
    </poi>
  </poi-set>'));

COMMIT;


-- --------------
-- XPath-ANFRAGEN
-- --------------

SET LONG 2000

--- existNode

-- Aufruf als Methode:
SELECT gkz FROM GemeindePOIs g
WHERE g.poi_xml.existsNode('//Point[./pos/text()="8.21271 53.13930"]') = 1;

-- Aufruf als Funktion:
SELECT existsNode(poi_xml,'//poi/pos') FROM GemeindePOIs;

--- extract

-- Aufruf als Methode:
SELECT g.poi_xml.extract('//Point') FROM GemeindePOIs g;

-- Aufruf als Funktion:
SELECT COUNT(extract(poi_xml,'//Point')) FROM GemeindePOIs;

-- Nutzung von getStringVal():
SELECT gkz FROM GemeindePOIs g
WHERE g.poi_xml.extract('/poi-set/poi[3]/name/text()').getStringVal() = 'Bahnhof';

SELECT extract(poi_xml,'//pos/text()[../../../name/text()="Bahnhof"]')
FROM GemeindePOIs;


-- -------------
-- GML-ERZEUGUNG
-- -------------

-- Java fr GML-Erzeugung

-- von KOMMANDOZEILE aus Java laden:
loadjava -user <Benutzer>/<Passwort>@<Dienstname> -resolve sdoapi.jar
loadjava -user <Benutzer>/<Passwort>@<Dienstname> -andresolve GenerateGML.java

-- Java-Methode bekanntgeben:
CREATE OR REPLACE FUNCTION gml (geom MDSYS.SDO_GEOMETRY) RETURN VARCHAR2 AS
   LANGUAGE JAVA NAME 'GenerateGML.toGML (oracle.sql.STRUCT)
                                                  return java.lang.String';
/

-- Methode nutzen:
SELECT name, gml(geo) FROM GeoDbLand;


-- ------------
-- GML-ANFRAGEN
-- ------------

--- Positionstabelle anlegen:
CREATE TABLE POIPositionen (
  gkz   INTEGER,              -- Gemeindekennzeichen
  name  VARCHAR(40),          -- POI-Name
  pos   MDSYS.SDO_GEOMETRY,   -- Position
  CONSTRAINT fk_pospoi_gkz FOREIGN KEY (gkz)  -- Fremdschlssel 
     REFERENCES GemeindePOIs(gkz) ON DELETE CASCADE
);

--- Metadaten bekanntgeben:
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('POIPOSITIONEN','pos', MDSYS.SDO_DIM_ARRAY(
           MDSYS.SDO_DIM_ELEMENT('Lnge', -180,180, 10),
           MDSYS.SDO_DIM_ELEMENT('Breite', -90,90, 10)), 8307);
COMMIT;

--- Index anlegen:
CREATE INDEX POIPositionen_pos_ix ON POIPositionen(pos)
INDEXTYPE IS MDSYS.SPATIAL_INDEX PARAMETERS('LAYER_GTYPE=POINT');

--- Punkte einfgen (aufgrund eingeschrnkter XML-Fhigkeiten "per Hand"):
INSERT INTO POIPositionen VALUES (3403000,'Schloss',
	MDSYS.SDO_GEOMETRY(2001,8307,MDSYS.SDO_POINT_TYPE(8.21271, 53.1393, NULL),NULL,NULL));
INSERT INTO POIPositionen VALUES (3403000,'FH Oldenburg',
	MDSYS.SDO_GEOMETRY(2001,8307,MDSYS.SDO_POINT_TYPE(8.19974, 53.14253, NULL),NULL,NULL));
INSERT INTO POIPositionen VALUES (3403000,'Bahnhof',
	MDSYS.SDO_GEOMETRY(2001,8307,MDSYS.SDO_POINT_TYPE(8.22025, 53.14416, NULL),NULL,NULL));

-- Rumliche Anfrage:
SELECT g.poi_xml.extract('//poi[./name/text()="' || r.name || '"]')
FROM GemeindePOIs g INNER JOIN 
  (SELECT p.gkz,name
   FROM POIPositionen p
   WHERE SDO_RELATE (pos, SDO_CS.VIEWPORT_TRANSFORM(
                           MDSYS.SDO_GEOMETRY(2003,0,null,
                             MDSYS.SDO_ELEM_INFO_ARRAY(1,1003,3),
                             MDSYS.SDO_ORDINATE_ARRAY(8.20,53, 9,54)), 8307),
                     'mask=ANYINTERACT, querytype=WINDOW') = 'TRUE')  r
ON g.gkz = r.gkz;
